@@ -2,7 +2,7 @@
  * input_ext-dec.h: structures exported to the VideoLAN decoders
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: input_ext-dec.h,v 1.28 2001/04/16 12:34:28 asmax Exp $
+ * $Id: input_ext-dec.h,v 1.32 2001/05/01 12:22:18 sam Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Michel Kaempf <maxx@via.ecp.fr>
@@ -169,7 +169,7 @@ typedef struct bit_stream_s
     byte_t *                p_byte;
     /* Pointer to the last byte that is to be read (in the current packet */
     byte_t *                p_end;
-    /* Temporary buffer in case we're not aligned when changing data packets. */
+    /* Temporary buffer in case we're not aligned when changing data packets */
     WORD_TYPE               i_showbits_buffer;
     data_packet_t           showbits_data;
 } bit_stream_t;
@@ -500,7 +500,10 @@ typedef struct decoder_config_s
     struct stream_ctrl_s *  p_stream_ctrl;
     struct decoder_fifo_s * p_decoder_fifo;
     void                 (* pf_init_bit_stream)( struct bit_stream_s *,
-                                                 struct decoder_fifo_s * );
+                                                 struct decoder_fifo_s *,
+                 void (* pf_bitstream_callback)( struct bit_stream_s *,
+                                                 boolean_t ),
+                                                 void * );
 } decoder_config_t;
 
 /*****************************************************************************
@@ -512,8 +515,6 @@ struct vout_thread_s;
 
 typedef struct vdec_config_s
 {
-    struct vout_thread_s *  p_vout;
-
     struct picture_s *   (* pf_create_picture)( struct vout_thread_s *,
                                                 int i_type, int i_width,
                                                 int i_height );
@@ -542,15 +543,10 @@ typedef struct vdec_config_s
  *****************************************************************************
  * Pointers given to audio decoders threads.
  *****************************************************************************/
-struct aout_thread_s;
-
 typedef struct adec_config_s
 {
-    struct aout_thread_s *  p_aout;
-
-    struct aout_fifo_s * (* pf_create_fifo)( struct aout_thread_s *,
-                                            struct aout_fifo_s * );
-    void                 (* pf_destroy_fifo)( struct aout_thread_s *);
+    struct aout_fifo_s * (* pf_create_fifo)( struct aout_fifo_s * );
+    void                 (* pf_destroy_fifo)( void );
 
     decoder_config_t        decoder_config;
 } adec_config_t;
