@@ -2,6 +2,7 @@
  * dvd.c : DVD input module for vlc
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
+ * $Id: dvd.c,v 1.10 2001/05/31 03:57:54 sam Exp $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -26,9 +27,9 @@
 #include "defs.h"
 
 #ifdef HAVE_CSS
-#define MODULE_NAME dvd-css
-#else /* HAVE_CSS */
 #define MODULE_NAME dvd
+#else /* HAVE_CSS */
+#define MODULE_NAME dvdnocss
 #endif /* HAVE_CSS */
 #include "modules_inner.h"
 
@@ -41,14 +42,7 @@
 #include "mtime.h"
 
 #include "modules.h"
-
-/*****************************************************************************
- * Build configuration tree.
- *****************************************************************************/
-MODULE_CONFIG_START
-ADD_WINDOW( "Configuration for DVD module" )
-    ADD_COMMENT( "foobar !" )
-MODULE_CONFIG_END
+#include "modules_export.h"
 
 /*****************************************************************************
  * Capabilities defined in the other files.
@@ -56,63 +50,47 @@ MODULE_CONFIG_END
 void _M( input_getfunctions )( function_list_t * p_function_list );
 
 /*****************************************************************************
- * InitModule: get the module structure and configuration.
- *****************************************************************************
- * We have to fill psz_name, psz_longname and psz_version. These variables
- * will be strdup()ed later by the main application because the module can
- * be unloaded later to save memory, and we want to be able to access this
- * data even after the module has been unloaded.
+ * Build configuration tree.
  *****************************************************************************/
-MODULE_INIT
-{
-    p_module->psz_name = MODULE_STRING;
+MODULE_CONFIG_START
+ADD_WINDOW( "Configuration for DVD module" )
+    ADD_COMMENT( "foobar !" )
+MODULE_CONFIG_STOP
+
+MODULE_INIT_START
+    p_module->i_capabilities = MODULE_CAPABILITY_NULL
+                                | MODULE_CAPABILITY_INPUT;
 #ifdef HAVE_CSS
     p_module->psz_longname = "full DVD input module with CSS decryption";
 #else /* HAVE_CSS */
     p_module->psz_longname = "DVD input module, CSS decryption disabled";
 #endif /* HAVE_CSS */
-    p_module->psz_version = VERSION;
-
-    p_module->i_capabilities = MODULE_CAPABILITY_NULL
-                                | MODULE_CAPABILITY_INPUT;
-
-    return( 0 );
-}
-
-/*****************************************************************************
- * ActivateModule: set the module to an usable state.
- *****************************************************************************
- * This function fills the capability functions and the configuration
- * structure. Once ActivateModule() has been called, the i_usage can
- * be set to 0 and calls to NeedModule() be made to increment it. To unload
- * the module, one has to wait until i_usage == 0 and call DeactivateModule().
- *****************************************************************************/
-MODULE_ACTIVATE
-{
-    p_module->p_functions = malloc( sizeof( module_functions_t ) );
-    if( p_module->p_functions == NULL )
-    {
-        return( -1 );
-    }
+MODULE_INIT_STOP
 
+MODULE_ACTIVATE_START
     _M( input_getfunctions )( &p_module->p_functions->input );
+MODULE_ACTIVATE_STOP
 
-    p_module->p_config = p_config;
+MODULE_DEACTIVATE_START
+MODULE_DEACTIVATE_STOP
 
-    return( 0 );
-}
-
-/*****************************************************************************
- * DeactivateModule: make sure the module can be unloaded.
- *****************************************************************************
- * This function must only be called when i_usage == 0. If it successfully
- * returns, i_usage can be set to -1 and the module unloaded. Be careful to
- * lock usage_lock during the whole process.
- *****************************************************************************/
-MODULE_DEACTIVATE
+#ifdef HAVE_CSS
+#else /* HAVE_CSS */
+#ifdef BUILTIN
+int module_dvd_InitModule( module_t *p_module )
 {
-    free( p_module->p_functions );
-
-    return( 0 );
+    return module_dvdnocss_InitModule( p_module );
 }
 
+int module_dvd_ActivateModule( module_t *p_module )
+{
+    return module_dvdnocss_ActivateModule( p_module );
+}
+
+int module_dvd_DeactivateModule( module_t *p_module )
+{
+    return module_dvdnocss_DeactivateModule( p_module );
+}
+#endif /* BUILTIN */
+#endif /* HAVE_CSS */
+
