@@ -2,6 +2,7 @@
  * input_dvd.h: thread structure of the DVD plugin
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
+ * $Id: input_dvd.h,v 1.22 2001/05/19 00:39:30 stef Exp $
  *
  * Author: Stphane Borel <stef@via.ecp.fr>
  *
@@ -33,33 +34,35 @@ typedef struct thread_dvd_data_s
 {
     int                     i_fd;               // File descriptor of device
     boolean_t               b_encrypted;        // CSS encryption
-    int                     i_read_once;        // NB of bytes read by DVDRead
 
-    int                     i_start_byte;
-    int                     i_start_cell;
-    int                     i_end_cell;
+    int                     i_block_once;       // Nb of block read once by 
+                                                // readv
+
+    /* Navigation information */
+    int                     i_title;
+    int                     i_title_id;
+
+    int                     i_chapter_nb;
+    int                     i_chapter;
+
+    int                     i_angle_nb;
+    int                     i_angle;
+    int                     i_angle_cell; /* cell index in the current angle */
+
+    int                     i_cell;              /* cell index in adress map */
+    int                     i_prg_cell;         /* cell index in program map */
+
+    int                     i_sector;
+    int                     i_end_sector;     /* last sector of current cell */
+
+    off_t                   i_title_start;
+    off_t                   i_start;
+    off_t                   i_size;
 
     /* Scrambling Information */
-    struct css_s            css;
+    struct css_s *          p_css;
 
     /* Structure that contains all information of the DVD */
-    struct ifo_s            ifo;
+    struct ifo_s *          p_ifo;
 
 } thread_dvd_data_t;
-
-/*****************************************************************************
- * Prototypes in dvd_ifo.c
- *****************************************************************************/
-struct ifo_s    IfoInit( int );
-int             IfoReadVTS( struct ifo_s * );
-void            IfoRead( struct ifo_s * );
-void            IfoEnd( ifo_t * );
-
-/*****************************************************************************
- * Prototypes in dvd_css.c
- *****************************************************************************/
-int             CSSTest     ( int );
-struct css_s    CSSInit     ( int );
-int             CSSGetKey   ( struct css_s * );
-int             CSSDescrambleSector( u8 * , u8 * );
-
