@@ -2,15 +2,15 @@
  * input_ts.c: TS demux and netlist management
  *****************************************************************************
  * Copyright (C) 1998, 1999, 2000 VideoLAN
- * $Id: input_ts.c,v 1.5 2001/02/20 07:49:13 sam Exp $
+ * $Id: input_ts.c,v 1.26 2001/06/03 12:47:21 sam Exp $
  *
- * Authors: 
+ * Authors: Henri Fallon <henri@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -32,19 +32,41 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/uio.h>
+
+#ifdef STRNCASECMP_IN_STRINGS_H
+#   include <strings.h>
+#endif
 
 #include <sys/types.h>
+
+#if !defined( _MSC_VER )
+#   include <sys/time.h>
+#endif
+
+#ifdef SYS_NTO
+#   include <sys/select.h>
+#endif
+
 #include <sys/stat.h>
-#include <unistd.h>
+
+#ifdef HAVE_UNISTD_H
+#   include <unistd.h>
+#endif
+
 #include <fcntl.h>
 
+#if defined( WIN32 )
+#   include <io.h>
+#   include "input_iovec.h"
+#else
+#   include <sys/uio.h>                                      /* struct iovec */
+#endif
+
 #include "config.h"
 #include "common.h"
 #include "threads.h"
 #include "mtime.h"
 #include "tests.h"
-#include "modules.h"
 
 #include "intf_msg.h"
 
@@ -58,14 +80,18 @@
 #include "mpeg_system.h"
 #include "input_netlist.h"
 
+#include "modules.h"
+#include "modules_export.h"
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
 static int  TSProbe     ( probedata_t * );
+static void TSInit      ( struct input_thread_s * );
+static void TSFakeOpen  ( struct input_thread_s * );
+static void TSEnd       ( struct input_thread_s * );
 static int  TSRead      ( struct input_thread_s *,
                           data_packet_t * p_packets[INPUT_READ_ONCE] );
-static void TSInit      ( struct input_thread_s * );
-static void TSEnd       ( struct input_thread_s * );
 
 /*****************************************************************************
  * Functions exported as capabilities. They are declared as static so that
@@ -76,8 +102,8 @@ void _M( input_getfunctions )( function_list_t * p_function_list )
 #define input p_function_list->functions.input
     p_function_list->pf_probe = TSProbe;
     input.pf_init             = TSInit;
-    input.pf_open             = input_FileOpen;
-    input.pf_close            = input_FileClose;
+    input.pf_open             = TSFakeOpen;
+    input.pf_close            = NULL;              /* Will be set by pf_open */
     input.pf_end              = TSEnd;
     input.pf_set_area         = NULL;
     input.pf_read             = TSRead;
@@ -107,12 +133,25 @@ static int TSProbe( probedata_t * p_data )
         return( 999 );
     }
 
+    if( ( strlen(psz_name) > 3 ) && !strncasecmp( psz_name, "ts:", 3 ) )
+    {
+        /* If the user specified "ts:" then it's probably a network stream */
+        return( 999 );
+    }
+
     if( ( strlen(psz_name) > 5 ) && !strncasecmp( psz_name, "file:", 5 ) )
     {
         /* If the user specified "file:" then it's probably a file */
         psz_name += 5;
     }
 
+    if( ( strlen(psz_name) > 3 ) &&
+                    !strncasecmp( psz_name+strlen(psz_name)-3, ".ts", 3) )
+    {
+        /* If it is a ".ts" file it's probably a TS file ... */
+        return( 900 );
+    }
+
     i_handle = open( psz_name, 0 );
     if( i_handle == -1 )
     {
@@ -130,57 +169,78 @@ static void TSInit( input_thread_t * p_input )
 {
     /* Initialize netlist and TS structures */
     thread_ts_data_t    * p_method;
-    pgrm_ts_data_t      * p_pgrm_demux;
-    es_descriptor_t     * kludge1;
+    es_descriptor_t     * p_pat_es;
+    es_ts_data_t        * p_demux_data;
+    stream_ts_data_t    * p_stream_data;
 
     /* Initialise structure */
     p_method = malloc( sizeof( thread_ts_data_t ) );
     if( p_method == NULL )
     {
-        intf_ErrMsg( "Out of memory" );
+        intf_ErrMsg( "TS input : Out of memory" );
         p_input->b_error = 1;
         return;
     }
- 
+
     p_input->p_plugin_data = (void *)p_method;
     p_input->p_method_data = NULL;
- 
-    
-    /* XXX : For the time being, only file, after, i'll do the network */ 
-    
+
+
     /* Initialize netlist */
-    if( input_NetlistInit( p_input, NB_DATA, NB_PES, TS_PACKET_SIZE, 
+    if( input_NetlistInit( p_input, NB_DATA, NB_PES, TS_PACKET_SIZE,
                 INPUT_READ_ONCE ) )
     {
-        intf_ErrMsg( "Could not initialize netlist" );
+        intf_ErrMsg( "TS input : Could not initialize netlist" );
         return;
     }
-   
+
     /* Initialize the stream */
     input_InitStream( p_input, sizeof( stream_ts_data_t ) );
 
-    /* FIXME : PSIDemux and PSIDecode */
-    /* Add audio and video programs */
-    /* p_input->stream.pp_programs[0] = */
-    input_AddProgram( p_input, 0, sizeof( pgrm_ts_data_t ) );
-    p_pgrm_demux = 
-        (pgrm_ts_data_t *)p_input->stream.pp_programs[0]->p_demux_data;
-    p_pgrm_demux->i_pcr_pid = 0x78;
+    /* input method type */
+    /* FIXME: should test if you have network or file here */
+    p_input->stream.i_method = INPUT_METHOD_NETWORK;
+    p_input->stream.p_selected_area->i_tell = 0;
 
-    kludge1 = input_AddES( p_input, p_input->stream.pp_programs[0], 
-                           0x78, sizeof( es_ts_data_t ) );
+    /* Init */
+    p_stream_data = (stream_ts_data_t *)p_input->stream.p_demux_data;
+    p_stream_data->i_pat_version = PAT_UNINITIALIZED ;
 
-    // kludge
-    kludge1->i_type = MPEG2_VIDEO_ES;
+    /* We'll have to catch the PAT in order to continue
+     * Then the input will catch the PMT and then the others ES
+     * The PAT es is indepedent of any program. */
+    p_pat_es = input_AddES( p_input, NULL,
+                           0x00, sizeof( es_ts_data_t ) );
+    p_demux_data=(es_ts_data_t *)p_pat_es->p_demux_data;
+    p_demux_data->b_psi = 1;
+    p_demux_data->i_psi_type = PSI_IS_PAT;
+    p_demux_data->p_psi_section = malloc(sizeof(psi_section_t));
+    p_demux_data->p_psi_section->b_is_complete = 1;
 
-    input_SelectES( p_input, kludge1 );
+}
 
-    vlc_mutex_lock( &(p_input->stream.stream_lock) );
-    p_input->stream.pp_programs[0]->b_is_ok = 1;
-    vlc_mutex_unlock( &(p_input->stream.stream_lock) );
+/*****************************************************************************
+ * TSFakeOpen: open the stream and set pf_close
+ *****************************************************************************/
+void TSFakeOpen( input_thread_t * p_input )
+{
+#if !defined( SYS_BEOS ) && !defined( SYS_NTO )
+    char *psz_name = p_input->p_source;
 
-//debug
-intf_ErrMsg("End of TSINIT");    
+    if( ( strlen(psz_name) > 3 ) && !strncasecmp( psz_name, "ts:", 3 ) )
+    {
+        /* If the user specified "ts:" he wants a network stream */
+        p_input->pf_open  = p_input->pf_network_open;
+        p_input->pf_close = p_input->pf_network_close;
+    }
+    else
+#endif
+    {
+        p_input->pf_open  = p_input->pf_file_open;
+        p_input->pf_close = p_input->pf_file_close;
+    }
+
+    p_input->pf_open( p_input );
 }
 
 /*****************************************************************************
@@ -188,7 +248,13 @@ intf_ErrMsg("End of TSINIT");
  *****************************************************************************/
 static void TSEnd( input_thread_t * p_input )
 {
+    es_descriptor_t     * p_pat_es;
 
+    p_pat_es = input_FindES( p_input, 0x00 );
+
+    if( p_pat_es != NULL )
+        input_DelES( p_input, p_pat_es );
+    free(p_input->p_plugin_data);
 }
 
 /*****************************************************************************
@@ -200,34 +266,69 @@ static void TSEnd( input_thread_t * p_input )
 static int TSRead( input_thread_t * p_input,
                    data_packet_t * pp_packets[INPUT_READ_ONCE] )
 {
-    unsigned int i_read, i_loop;
-    struct iovec * p_iovec;
-    
-    memset( pp_packets, 0, INPUT_READ_ONCE*sizeof(data_packet_t *) );
-    
+    thread_ts_data_t    * p_method;
+    unsigned int    i_read, i_loop;
+    int             i_data;
+    struct iovec  * p_iovec;
+    struct timeval  s_wait;
+
+
+    /* Get iovecs */
     p_iovec = input_NetlistGetiovec( p_input->p_method_data );
-    
+
     if ( p_iovec == NULL )
     {
         return( -1 ); /* empty netlist */
-    } 
+    }
 
-    i_read = readv( p_input->i_handle, p_iovec, INPUT_READ_ONCE );
-    
-    if( i_read == -1 )
+    /* Init */
+    p_method = ( thread_ts_data_t * )p_input->p_plugin_data;
+
+    /* Initialize file descriptor set */
+    FD_ZERO( &(p_method->fds) );
+    FD_SET( p_input->i_handle, &(p_method->fds) );
+
+    /* We'll wait 0.5 second if nothing happens */
+    s_wait.tv_sec = 0;
+    s_wait.tv_usec = 500000;
+
+    /* Reset pointer table */
+    memset( pp_packets, 0, INPUT_READ_ONCE * sizeof(data_packet_t *) );
+
+    /* Fill if some data is available */
+    i_data = select( p_input->i_handle + 1, &(p_method->fds), NULL, NULL,
+                     &s_wait);
+
+    if( i_data == -1 )
     {
-        intf_ErrMsg( "Could not readv" );
+        intf_ErrMsg( "input error: TS select error (%s)", strerror(errno) );
         return( -1 );
     }
 
-    input_NetlistMviovec( p_input->p_method_data, 
-            (int)(i_read/TS_PACKET_SIZE) , pp_packets );
-
-    /* check correct TS header */
-    for( i_loop=0; i_loop < (int)(i_read/TS_PACKET_SIZE); i_loop++ )
+    if( i_data )
     {
-        if( pp_packets[i_loop]->p_buffer[0] != 0x47 )
-            intf_ErrMsg( "Bad TS Packet (starcode != 0x47)." );
+        i_read = readv( p_input->i_handle, p_iovec, INPUT_READ_ONCE );
+
+        if( i_read == -1 )
+        {
+            intf_ErrMsg( "input error: TS readv error" );
+            return( -1 );
+        }
+
+        input_NetlistMviovec( p_input->p_method_data,
+                (int)(i_read/TS_PACKET_SIZE) , pp_packets );
+
+        /* check correct TS header */
+        for( i_loop=0; i_loop * TS_PACKET_SIZE < i_read; i_loop++ )
+        {
+            if( pp_packets[i_loop]->p_buffer[0] != 0x47 )
+                intf_ErrMsg( "input error: bad TS packet (starts with "
+                             "0x%.2x, should be 0x47)",
+                             pp_packets[i_loop]->p_buffer[0] );
+        }
+
+        p_input->stream.p_selected_area->i_tell += i_read;
     }
     return 0;
 }
+
