@@ -2,7 +2,7 @@
  * ac3_decoder.h : ac3 decoder interface
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: ac3_decoder.h,v 1.4 2001/03/21 13:42:34 sam Exp $
+ * $Id: ac3_decoder.h,v 1.9 2001/05/15 16:19:42 sam Exp $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
  *          Renaud Dartus <reno@videolan.org>
@@ -32,22 +32,12 @@ typedef struct ac3_sync_info_s {
     int bit_rate;       /* nominal bit rate in kbps */
 } ac3_sync_info_t;
 
-typedef struct ac3_byte_stream_s {
-    u8 * p_byte;
-    u8 * p_end;
-    void * info;
-} ac3_byte_stream_t;
 
 /**** ac3 decoder API - functions publically provided by the ac3 decoder ****/
 
 int ac3_init (ac3dec_t * p_ac3dec);
 int ac3_sync_frame (ac3dec_t * p_ac3dec, ac3_sync_info_t * p_sync_info);
 int ac3_decode_frame (ac3dec_t * p_ac3dec, s16 * buffer);
-static ac3_byte_stream_t * ac3_byte_stream (ac3dec_t * p_ac3dec);
-
-/**** ac3 decoder API - user functions to be provided to the ac3 decoder ****/
-
-void ac3_byte_stream_next (ac3_byte_stream_t * p_byte_stream);
 
 /**** EVERYTHING AFTER THIS POINT IS PRIVATE ! DO NOT USE DIRECTLY ****/
 
@@ -275,7 +265,7 @@ typedef struct audblk_s {
 //      u16 chmant[5][256];
 
     /* coupling mantissas */
-    float cplfbw[ 256 ];
+    float cpl_flt[ 256 ];
 //      u16 cplmant[256];
 
     /* coupling mantissas */
@@ -318,6 +308,7 @@ typedef struct audblk_s {
 } audblk_t;
 
 /* Everything you wanted to know about band structure */
+
 /*
  * The entire frequency domain is represented by 256 real
  * floating point fourier coefficients. Only the lower 253
@@ -336,26 +327,6 @@ typedef struct audblk_s {
  * approximate a 1/6 octave scale.
  */
 
-typedef struct stream_coeffs_s
-{
-    float fbw[5][256];
-    float lfe[256];
-} stream_coeffs_t;
-
-typedef struct stream_samples_s
-{
-    float channel[6][256];
-} stream_samples_t;
-
-typedef struct ac3_bit_stream_s
-{
-    u32 buffer;
-    int i_available;
-    ac3_byte_stream_t byte_stream;
-
-    unsigned int total_bits_read; /* temporary */
-} ac3_bit_stream_t;
-
 typedef struct bit_allocate_s
 {
     s16 psd[256];
@@ -381,38 +352,6 @@ typedef struct mantissa_s
     u16 lfsr_state;
 } mantissa_t;
 
-typedef struct complex_s {
-    float real;
-    float imag;
-} complex_t;
-
-#define N 512
-
-typedef struct imdct_s
-{
-    complex_t buf[N/4];
-
-    /* Delay buffer for time domain interleaving */
-    float delay[6][256];
-
-    /* Twiddle factors for IMDCT */
-    float xcos1[N/4];
-    float xsin1[N/4];
-    float xcos2[N/8];
-    float xsin2[N/8];
-    
-    /* Twiddle factor LUT */
-    complex_t *w[7];
-    complex_t w_1[1];
-    complex_t w_2[2];
-    complex_t w_4[4];
-    complex_t w_8[8];
-    complex_t w_16[16];
-    complex_t w_32[32];
-    complex_t w_64[64];
-
-} imdct_t;
-
 struct ac3dec_s
 {
     /*
@@ -420,8 +359,10 @@ struct ac3dec_s
      */
 
     /* The bit stream structure handles the PES stream at the bit level */
-    ac3_bit_stream_t    bit_stream;
-
+    bit_stream_t        bit_stream;
+    int                 i_available;
+    unsigned int        total_bits_read; /* temporary */
+                
     /*
      * Decoder properties
      */
@@ -429,17 +370,13 @@ struct ac3dec_s
     bsi_t               bsi;
     audblk_t            audblk;
 
-    stream_coeffs_t     coeffs;
-    stream_samples_t    samples;
+    float               samples[6][256];
+    dm_par_t            dm_par;
 
     bit_allocate_t      bit_allocate;
     mantissa_t          mantissa;
     imdct_t             imdct;
+    downmix_t           downmix;
+
 };
 
-/**** ac3 decoder inline functions ****/
-
-static ac3_byte_stream_t * ac3_byte_stream (ac3dec_t * p_ac3dec)
-{
-    return &(p_ac3dec->bit_stream.byte_stream);
-}
